<?php
use JetBrains\PhpStorm\NoReturn;

require 'app-config.php';

session_start([
    'name'            => 'FRCSESSION',
    'use_strict_mode' => true,
    'cookie_httponly' => true,
    'cookie_samesite' => 'Strict']);

/**
 * Add a message to be displayed at the top of the page
 *
 * @param string $level The level (type) of the message
 * @param string $message The message itself
 */
function add_message(string $level, string $message): void {
    if (!array_key_exists(Key::USER_MSG, $_SESSION)) $_SESSION[Key::USER_MSG] = array();
    $_SESSION[Key::USER_MSG][] = ['level' => $level, 'message' => $message];
}

/**
 * Add an error message to be displayed at the top of the page
 *
 * @param string $message The message to be displayed
 */
function add_error(string $message): void {
    add_message('ERROR', $message);
}

/**
 * Add an error message to be displayed at the top of the page
 *
 * @param string $message The message to be displayed
 */
function add_info(string $message): void {
    add_message('INFO', $message);
}

/** @var bool $is_htmx True if this request was initiated by htmx, false if not */
$is_htmx = array_key_exists('HTTP_HX_REQUEST', $_SERVER)
    && !array_key_exists('HTTP_HX_HISTORY_RESTORE_REQUEST', $_SERVER);

/**
 * Render the page title
 * @param string $title The title of the page being displayed
 */
function page_head(string $title): void {
    global $is_htmx;
    $version = match (true) {
        str_ends_with(FRC_VERSION, '.0.0') => substr(FRC_VERSION, 0, strlen(FRC_VERSION) - 4),
        str_ends_with(FRC_VERSION, '.0')   => substr(FRC_VERSION, 0, strlen(FRC_VERSION) - 2),
        default                            => FRC_VERSION
    };
    //if ($is_htmx) header('HX-Push-Url: true');
    ?>
<!DOCTYPE html>
<html lang=en>
<head>
    <title><?=$title?> | Feed Reader Central</title><?php
    if (!$is_htmx) { ?>
        <meta name=viewport content="width=device-width, initial-scale=1">
        <link href=/assets/style.css rel=stylesheet><?php
    } ?>
</head>
<body><?php
    if (!$is_htmx) { ?>
        <header hx-target=#main hx-push-url=true>
            <div><a class=title href=/ hx-get="/">Feed Reader Central</a><span class=version>v<?=$version?></span></div>
            <div><?php
                if (array_key_exists(Key::USER_ID, $_SESSION)) { ?>
                    <a href=/feed?id=new hx-get=/feed?id=new>Add Feed</a> |
                    <a href=/docs/ hx-get=/docs/>Docs</a> |
                    <a href=/user/log-off hx-get=/user/log-off>Log Off</a><?php
                    if ($_SESSION[Key::USER_EMAIL] != Security::SINGLE_USER_EMAIL) { ?>
                        | <?=$_SESSION[Key::USER_EMAIL]?><?php
                    }
                } else { ?>
                    <a href=/user/log-on hx-get=/user/log-on>Log On</a> | <a href=/docs/ hx-get=/docs/>Docs</a><?php
                } ?>
            </div>
        </header>
        <main id=main hx-target=this hx-push-url=true hx-swap="innerHTML show:window:top"><?php
    }
        if (sizeof($messages = $_SESSION[Key::USER_MSG] ?? []) > 0) { ?>
            <div class=user_messages><?php
                array_walk($messages, function ($msg) { ?>
                    <div class=user_message>
                        <?=$msg['level'] == 'INFO' ? '' : "<strong>{$msg['level']}</strong><br>"?>
                        <?=$msg['message']?>
                    </div><?php
                }); ?>
            </div><?php
            $_SESSION[Key::USER_MSG] = [];
        }
}

/**
 * Render the end of the page
 */
function page_foot(): void {
    global $is_htmx; ?>
    </main><?php
    if (!$is_htmx) { ?>
        <script src=/assets/htmx.min.js></script><?php
    } ?>
</body>
</html><?php
    session_commit();
}

/**
 * Redirect the user to the given URL
 *
 * @param string $value A local URL to which the user should be redirected
 */
#[NoReturn]
function frc_redirect(string $value): void {
    if (str_starts_with($value, 'http')) {
        http_response_code(400);
        die();
    }
    session_commit();
    header("Location: $value", true, 303);
    die();
}

/**
 * Convert a date/time string to a date/time in the configured format
 *
 * @param string $value The date/time string
 * @return string The standard format of a date/time, or '(invalid date)' if the date could not be parsed
 */
function date_time(string $value): string {
    try {
        return (new DateTimeImmutable($value))->format(DATE_TIME_FORMAT);
    } catch (Exception) {
        return '(invalid date)';
    }
}
